from .utils import error_suppress, connect_false, false
import requests
import json
from sseclient import SSEClient


class EchoerHandler:
    def __init__(self, url):
        self.action = []
        self.model = []
        self.success_status = []
        self.fail_status = []
        self.url = url
        self.version = 0
        self.first_connect()

    @staticmethod
    def get_flow_name(data):
        with error_suppress(false):
            name = '_'.join(data.split('_')[:-1])
            return name

    def first_connect(self):

        with error_suppress(connect_false):
            data = requests.get(f'{self.url}/flowrun')
            self.version = int(data.json()[-1]['metadata']['version']) - 100

    def add_action(self, action, model, success, fail):
        self.action.append(action)
        self.model.append(model)
        self.success_status.append(success)
        self.fail_status.append(fail)

    def artifact_connect(self):

        # sse长连接请求结果，将实时打印返回的数据响应
        messages = SSEClient(f'{self.url}/watch?resource=flowrun?version={self.version}')
        for msg in messages:
            if msg == "":
                print("返回数据为空")
                continue
            data = json.loads(msg.data)
            version = data['metadata']['version']

            for step in data['spec']['steps']:
                action = step['metadata']['name']
                print(f'检测到action{action}')
            self.version = int(version) - 1



from .utils import error_suppress, false
from .step import Step
import requests


class FlowRun:

    def __init__(self, req_url, name):
        self.req_url = req_url
        self._name = name
        self._step_list = []
        self._desc = []
        self._step = []

    def add_step(self, name, step, action, args):
        new_step = Step(name=name, action=action)
        desc = new_step.flow_return(step=step, args=args)
        self._step = list(set(self._step).union(set(desc)))
        self._step.sort()
        self._step_list.append(new_step)

    def check_desc(self, desc):
        if not set(self._step).issubset(set(desc.keys())):
            return False
        return True

    def generate(self):
        data = ''
        for step in self._step_list:
            _ = step.generate()
            data += f'{_}\n'
        return f'flow_run  {self._name}\n {data} flow_run_end'

    def send_echoer(self):
        a = self.generate()
        a = a.replace('"`', '`')
        a = a.replace('`"', '`')
        flow_data = {'data': a}
        print(flow_data['data'])

        with error_suppress(false):
            req = requests.post(self.req_url, json=flow_data, timeout=30)
            if req.status_code == 200:
                return True
            else:
                print('发送echoer错误', req.json())
                return False



class Step:

    def __init__(self, name, action):
        self._flow = None
        self._str_args = None
        self._desc = None

        self._name = name
        self._action = action

    def flow_return(self, step, args=None):
        flow = ''
        for key in step.keys():
            if flow:
                flow += ' | '
            flow += f'{key}->{step[key]}'

        str_args = ''
        if isinstance(args, dict):
            for key in args.keys():
                if str_args:
                    str_args += ','
                value = f'{args[key]}' if isinstance(args[key], int) else f'"{args[key]}"'
                str_args += f'{key}={value}'

        self._flow = flow
        self._str_args = str_args
        self._desc = list(step.values())
        return [self._name]

    def generate(self):
        _ = f'step {self._name} => ({self._flow}) {{action = "{self._action}"; args = ({self._str_args});}};'

        return _



import traceback
from contextlib import ContextDecorator


class error_suppress(ContextDecorator):

    def __init__(self, error_func, ):
        self.error_msg = error_func

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.error_msg()


def false():
    print('发送 echoer 错误', traceback.format_exc())
    return False


def connect_false():
    print('发送 echoer 错误', traceback.format_exc())



from .utils import error_suppress, false
import requests


class Action:
    def __init__(self, req_url, name):
        self.name = name
        self.req_url = req_url
        self.params = ''

    def add_data(self, dictData):
        if not isinstance(dictData, dict):
            return False
        length = len(dictData)
        params = '('
        for k, v in dictData.items():
            if str(v) != 'str' and str(v) != 'int':
                return False
            if length == 1:
                params += f'{v} {k})'
            else:
                params += f'{v} {k},'
            length -= 1
        self.params = params
        return True

    def generate(self):
        data = \
            f"""action {self.name}
                  addr = "{self.req_url}";
                  method = http;
                  args = {self.params};
                  return = (SUCCESS | FAIL);
                action_end
            """
        return data

    def send_echoer(self):
        data = self.generate()
        action_data = {"data": data}
        with error_suppress(false):
            req = requests.post(self.req_url, json=action_data, timeout=30)
            if req.status_code == 200:
                return True
            else:
                print('action发送echoer错误', req.json())
                return False
